# Bash Utility Functions

#####################################################################################
## normpath - Outputs the normalized (canonical) path to a file, similar to realpath, 
##   except that by default normpath does NOT resolve symlinks (although there is a 
##   flag that causes it to do so.) Furthermore (by default) the path need not exist.
##   This function is required mainly because 'realpath' is not present on many
##   systems and 'readlink' resolves symlinks, which can be undesirable.
##
##   * This outputs the absolute path to a file, unless the --rel flag has been
##     specified, in which case the function outputs a path that is relative to the
##     indicated directory.
##   * A 'base' folder can be specified (-b|--base), and it will be prepended to 
##     the subject path when attempting to normalize it (but only if the subject 
##     path is a relative path). 
##   
###############################################################################
## Usage: normpath path [OPTIONS] 
###############################################################################
##  OPTIONS:
##
##  -m|--sym            Resolve symlinks (default is to NOT resolve)
##
##  -s|--silent         Silences error messages about non-existant paths.
##
##  -e|--exists=EXISTS  Specify whether or not the path must exist.
##      Valid values for EXISTS are:
##      *  all  --  The entire resolved path must exist.
##      *  last --  All but the last component of the path must exist.
##      *  none --  No part of the path is required to exist. (default)
##      If the test for existance fails, the output is empty and the
##      exit status will be 1.
##
##  -r|--rel=REFDIR     The resulting output path will be made relative to 
##      REFDIR (even if REFDIR is not a parent directory.)
##
##  -b|--base=BASEDIR   Specify that the original path *might be* relative 
##      to BASEDIR. This causes BASEDIR to be prepended to 'path', but only
##      if 'path' is not absolute. If the path (or a portion of it) is 
##      required to exist (due to the --exists flag), and the test for
##      existance fails when prepending the base, then the function will
##      re-attempt the test without the base.
##
###############################################################################
function normpath() {
    local path relative_to baseDir="$PWD" 
    local silent=0 symlinks=0 exists=none
    local opt val
    local argPos=0

    for arg in "$@"; do 
        # an option is -n or --name or -n=value or --name=value  or -n:value or --name:value
        if [[ $arg == -* ]]; then
            opt=${arg%%[=:]*}  #opt keeps the hyphens!
            val=${arg:${#opt}+1}
        else
            # use the index of the positional argument as the option name e.g. 1, 2, 3 etc
            opt=$((++argPos))
            val="$arg"
        fi
        case $(tolower $opt) in #options are case NOT case sensitive (,, causes the variable to be lower case)
            1)                          path="$val"         ;;
            2|-r|--rel|--relative-to)   relative_to="$val"  ;;
            3|-b|--base)                baseDir="$val"      ;;
            -e|--exist|--exists)        exists="$val"       ;;
            -s|--silent)                silent=1            ;;
            -m|--sym)                   symlinks=1          ;;
            *) echo >&2 "Invalid flag passed to normpath: $arg"; return 1 ;;
        esac
    done

    while true; do #allows us to repeat the following if -base=BASEDIR fails the first time.
        # first get the absolute path
        local abspath="$path"
        [[ $path == /* ]] || abspath="$baseDir/$path"

        if ((symlinks)); then
            abspath=$(readlink -s -m -n "$abspath") # -s=silent -m=allow-missing -n=no-newline
        else
            abspath=$(cleanpath "$abspath")
        fi

        local error
        case "$exist" in
            all)        [[ -e $abspath ]]; error=$?         ;;
            no|none)    error=0                             ;;
            last)       local dir=$(dirname "$abspath")
                        [[ ! $dir || -e $dir ]]; error=$?   ;;
            *) echo >&2 "Invalid flag passed to normpath: $arg"
               return 1 ;; 
        esac

        if ((error)); then
            if [[ $baseDir == $PWD ]]; then
                # there's no point in retrying the loop because baseDir is already PWD.
                # just show the error and exit
                ((silent)) || echo >&2 "normpath: ‘$path’: No such file or directory"
                return 1
            else
                # the user-specified baseDir failed, so retry with PWD
                baseDir="$PWD"
            fi
        else
            # we have successfully found the path, so exit the loop
            break
        fi
    done

    if [[ $relative_to ]]; then 
        path=$(relpath "$path"  "$relative_to")
    fi

    echo "$path"
}

###############################################################################
# Usage: relpath subjectPath referencePath
###############################################################################
#   Given a target path (which is either absolute or relative to PWD)
#   and a base path, this outputs a relative path to the target 
#   from to the base path.
#
#   Examples:
#       relpath  /foo/bar/file  /foo/ ==> bar/file
#       relpath  /foo/bar/file  /foo/etc/ ==> ../bar/file
#       relpath  /usr/bin/exe   /home/user/ ==> ../../usr/bin/exe
###############################################################################
function relpath() {
    local base="$1"
    local target="$2"
    local base_parts=()
    local targ_parts=()

    #Split path elements & canonicalize
    local IFS='/'
    local bPos=0;
    for bp in $base; do
        case "$bp" in
            ".");;
            "..") ((bPos--)) ;;
            *) base_parts[$((bPos++))]="$bp" ;;
        esac
    done
    local tPos=0;
    for tp in $target; do
        case "$tp" in
            ".");;
            "..") ((tPos--)) ;;
            *) targ_parts[$((tPos++))]="$bp" ;;
        esac
    done
    #unset IFS

    #Count common prefix
    common=0
    for (( i=0 ; i<$bpl ; i++ )); do
        if [[ ${base_parts[$i]} == ${targ_parts[$common]} ]] ; then
            ((common++))
        else
            break
        fi
    done

    #Compute number of directories up
    local updir=$((bPos-common))

    #trivial case (after canonical decomposition)
    if [[ $updir -eq 0 ]]; then
        echo .
        return 0
    fi

    local result
    #Print updirs
    for (( i=0 ; i<$updir ; i++ )); do
        result+=../
    done

    #Print remaining path
    for (( i=$common ; i<$tPos ; i++ )); do
        if [[ ${targ_parts[$i]} ]]; then
            [[ $i == $common ]] || result+='/'
            result+="${targ_parts[$i]}"
        fi
    done

    echo "$result"
}

###############################################################################
##  usage: cleanpath PATH
##  Cleans up PATH by performing the following replacements:
##  * Replaces multiple adjacent slashes with a single slash:
##      PathA//pathB ==> PathA/PathB
##  * Replaces /./ with /
##      PathA/./PathB ==> /PathA/PathB
##  * Removes dir/.. 
##      PathA/PathB/../pathC  ==> /PathA/pathC
###############################################################################
function cleanpath() {
  # Replace all /./ sequences with /
  local path="${1//\/.\//\/}"

  # Paths can still start with ./ (and this is good)
  # But a path like ./../ should be converted to ../
  path="${path/#.\/./.}"  #i.e. ./. ==> .  (but only at the beginning)

  # Replace all // with /  (also ///, ////, ... )
  path="${path//\/+(\/)/\/}" #uses extglob:  +(pattern) means one or more occurrances

  # Remove dir/.. sequences.
  local A='*([^\/])' # pattern meaning 0 or more characters that are NOT slashes
  local B='[^\/.]' # pattern meaning a single character that is neither "/" nor "."
  local S=/ # slash -- useful in replacement patterns to replace a literal slash
  oldPath=
  while [[ $path != $oldPath ]]; do #repeat until path stops changing
    oldPath="$path"
    # replace PATH/../  with  ""
    # PATH must include NO slashes, and must not be ".."
    # require the ending slash to distinguish between ../ and ..word (which IS a valid file name)

    #  /hello/../  ==> /
    #  /hello/../world  ==> /world
    #  /hello/..world  ==> /hello/..world (no change)
    #  /hello/world/../../  ==> /hello/../
    #  /hello/world/../../path  ==> /hello/../path
    #  ../path  ==> ../path (no change)
    #  ../../path  ==> ../../path (no change)
    #  hello/../  ==> ""  (should this be an error?)
    path="${path/$A$B$A$S..$S/}" # replace "ABA/../" with ""
    
    # do not require the ending slash, but ONLY at the very end (%)
    #  /hello/..  ==> /
    #  /hello/../world/..  ==> /hello/../
    #  hello/..  ==> ""  (should this be an error?)
    path="${path/%$A$B$A$S../}" # replace "ABA/../" with ""
  done
  echo "$path"
}

###############################################################################
## Functions to deal with indirect references
###############################################################################
## isVar - return 0 if the variable name represented by $1 has been set (even
##      if it is set to NULL). If the variable is unset, 1 will be returned.
##      isVar $variableName && echo "$variableName is set!"
function isVar() { declare -p &>/dev/null "$1"; }
function setVar() { export -n "$1"="$2"; }
function getVar() { echo "${!1}"; }


###############################################################################
##  usage: var=$(tolower $var)
##  tolower, toupper: convert the input text to lower or upper case,
##     respectively and outputs the result.
##  This is required because bash < v4 lacks the ${VAR,,} or ${VAR^^} syntax.
###############################################################################
function tolower() { 
    # Re-define the function on first use, depending on bash version.
    if ((BASH_VERSINFO[0] < 4)); then
        function tolower() { echo "$*" | tr '[:upper:]' '[:lower:]'; }
    else
        function tolower() { echo "${*,,}"; }
    fi
    tolower "$*" # call the newly-defined version of the function
}
function toupper() { 
    # Re-define the function on first use, depending on bash version.
    if ((BASH_VERSINFO[0] < 4)); then
        function toupper() { echo "$*" | tr '[:lower:]' '[:upper:]'; }
    else
        function toupper() { echo "${*^^}"; }
    fi
    toupper "$*" # call the newly-defined version of the function
}

###############################################################################
##  usage: replaceInvalidVarChars <TEXT>
##  Replaces the following characters in TEXT with an unerscore (_)
##          - + @ = . ~ \ ! , ^ $ # % * " ' 
###############################################################################
function replaceInvalidVarChars() { echo "${1//[-+@ =.~\\!,^\$#%*\"\']/_}"; }

###############################################################################
##  usage: ifzero <TEST> <VALUE1> <VALUE2>
##  If TEST evaluates mathematically to 0, this outputs $VALUE1
##    Otherwise (i.e. TEST != 0) it outputs $VALUE2
###############################################################################
function ifzero() { (($1==0)) && echo $1 || echo $2; }

###############################################################################
##  usage:  trim [<TEXT>]   or   <COMMAND> | trim
##  Removes leading and trailing space from stdin or arguments.
###############################################################################
function trim() {
    local var
    if (($#)); then var="$*"; else var=$(cat); fi
    var="${var#"${var%%[![:space:]]*}"}"   # remove leading whitespace characters
    var="${var%"${var##*[![:space:]]}"}"   # remove trailing whitespace characters
    echo "$var"
}

###############################################################################
##  usage:  showVars [<VAR_NAME>...]
##  Prints out the variable names and their corresponding values, each on
##    a separate line.
###############################################################################
function showVars() {
    for var in "$@"; do echo -e "\t* \$"$var"\t"=" ""${!var}"; done
}

###############################################################################
##  usage:  joinArgs <DELIM> [<ARGS>...]
##  Joins all ARGS together as a single string using DELIM as a separator.
##  E.G.: joinArgs '|' apple 'banana bread' '' last  ==> "apple|banana bread||last"
###############################################################################
function joinArgs() { local IFS="$1"; echo "${*:2}"; }

###############################################################################
##  Usage:   if isWindows; then ...
##  The exit code is 0 (success) if the current operating system is Microsoft
##      Windows (as determined based on the OSTYPE environment variable set 
##      by the current shell).  Otherwise the exit code is 1 (failure).
###############################################################################
function isWindows() { [[ $OSTYPE == cygwin || $OSTYPE == msys ]]; }

###############################################################################
##  Usage:   if isMac; then ...
##  The exit code is 0 (success) if the current operating system is Mac OSX
##      (as determined based on the OSTYPE environment variable set 
##      by the current shell).  Otherwise the exit code is 1 (failure).
###############################################################################
function isMac() { [[ $OSTYPE == darwin* ]]; }

###############################################################################
##  Usage:   if isNix; then ...
##  The exit code is 0 (success) if the current operating system is Linux or  
##      similar *nix (as determined based on the OSTYPE environment variable set 
##      by the current shell).  Otherwise the exit code is 1 (failure).
###############################################################################
function isNix() { [[ $OSTYPE == linux* || $OSTYPE == bsd* || $OSTYPE == freebsd* || $OSTYPE == solaris* ]]; }

###############################################################################
##  Usage:  OS=$(getOS)
##  Outputs the canonicalized OS name, based on the OSTYPE bash variable.
##  Known OS list:  Windows, Linux, Mac, BSD, Solaris
###############################################################################
function getOS() {
  case $(tolower $OSTYPE) in
    cygwin|msys)    KNOWN_OS_NAME=Windows  ;;
    linux*)         KNOWN_OS_NAME=Linux    ;;
    darwin*)        KNOWN_OS_NAME=Mac      ;; 
    bsd*|freebsd*)  KNOWN_OS_NAME=BSD      ;;
    solaris*)       KNOWN_OS_NAME=Solaris  ;;
    *)              KNOWN_OS_NAME=UNKNOWN  ;;
  esac
  function getOS() { echo "$KNOWN_OS_NAME"; } # re-define for faster performance
  getOS # call re-defined function
}

###############################################################################
##  Usage: if is64Bit; then ...   OR   is64Bit && ...
##  The exit code is 0 (success) if the current shell is a 64-bit process (as 
##    reported by the HOSTTYPE environtment variable). 
##  Otherwise the exit code is 1 (failure).
###############################################################################
function is64Bit() {  [[ $HOSTTYPE == *64 ]]; }

###############################################################################
##  Usage: BITS=$(getArchBits) 
##  Outputs '32' or '64' depending on the architecture of the current shell/ENV.
###############################################################################
function getArchBits() {
  [[ $HOSTTYPE == *64 ]] && echo 64 || echo 32 
}

###############################################################################
##  Usage: if isEmptyDir <DIR>; then ...
##  Determines whether or not the specified directory is empty. The exit code
##    is 0 if the directory is empty or 1 if it contains any files or sub-
##    directories (other than . and .. )
##  If the specified directory does not exist (or is a file etc), an error
##    message is displayed and the exit code is 2
###############################################################################
function isEmptyDir() {
    [[ -d "$1" ]] && (($(ls -fq1 "$1"|wc -l)<=2))  || { printf >&2 'isEmptyDir: "%s" is not a directory.' "$1"; exit 2; }
}

###############################################################################
##  Usage: count=$(fileCount <DIR>)
##  Outputs the number of items (files or sub-directories) contained in the 
##    the specified directory. It does not count (. or ..)
##  If the specified directory does not exist (or is a file etc), the output
##    is 0.
###############################################################################
function fileCount() {
    # Count files in the directory
    [[ -d "$1" ]] && echo $(($(ls -fq1 "$1"|wc -l)-2)) || echo 0
}

###################### Shell Options (shopt) ##################################
# sopt  -- set, unset, or reset a shell option   (-s/-u/-r) respectively.
# When an option is set or unset, first save the previous value so it can be recurively reset. 
# Values are saved in a stack-like manner so that they can be recursively set and reset.
sopt() {
    local action=s  # set by default
    local val stackName stack size newval    
    [[ $1 == -[rsu] ]] && { action=${1:1}; shift; } # now action= s | u | r
    (($#)) || return 0 # no options to set
    for name in "$@"; do
        [[ -z $name ]] && continue
        shopt -q $name && val=s || val=u  # Get the current value of the option
        stackName=PREV_OPTS_$name   # The name of the stack variable containing previous values e.g. PREV_OPTS_extglob
        stack=${!stackName}         # get the current stack 
        size=${#stack}              # Get the size of the stack
        newval=$action              # The new shopt value (if action == r, newval will be changed to u or s)

        # The array does not exist, so create it (if $action is 'set' or 'unset'. Show an error if $action is 'reset')
        if [[ $action == r ]]; then
            # Action is RESET
            if ((size==0)); then
                echo >&2 "Error resetting option '$name': no previous value was saved."
                newval=u      # unset the value
            else
                newval=${stack:$size-1}  # Get the last value in the previous-value string
                stack=${stack:0:$size-1} # Remove the last character in the string
            fi
            #dbg: echo "Resetting $name to -$newval"
            shopt -$newval "$name"
        else
            # Action is SET or UNSET
            stack+=$val   # Add existing value to the end of the previous-value list
            # newval is not relevant. all options will be set together at the end
            #dbg: echo "$name: old=$val new=$action"
        fi
        export -n $stackName="$stack"
        echo -e "$stackName=\t$stack"
    done

    if [[ $action == [us] ]]; then
        #dbg: echo "shopt -$action $@"
        shopt -$action "$@"
    fi
}
usopt() { sopt -u "$@"; }
rsopt() { sopt -r "$@"; }
qopt() { for name in "$@"; do echo -n $name ''; shopt -q $name && echo on || echo off; done; }
##################################################################################

# apush - Push a value onto an array
# apush ARRNAME VALUE

#apush() { $1=("${$1[@]}" "$2"); }
#apop() { $1=(${$1[@]:0:${#$1[@]}-1}); }

########### Variable Save and Restore ##############################
