/*
 * (c) 2011 Mathews Lab, University of Rochester Medical Center.
 *
 * This software is part of a group specifically designed for the RNAstructure
 * secondary structure prediction program.
 */

package ur_rna.RNAstructureUI.windows;

import ur_rna.RNAstructureUI.AppMainFrame;
import ur_rna.RNAstructureUI.RNAstructureBackendCalculator;
import ur_rna.RNAstructureUI.menus.DynamicMenuBar;
import ur_rna.RNAstructureUI.menus.RolloverMenu;
import ur_rna.RNAstructureUI.utilities.OSInfo;

import javax.swing.*;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import java.awt.*;
import java.awt.event.*;

/**
 * A class that holds internal frames for the RNAstructure GUI.
 * <br><br>
 * Every frame is connected to the back end through an individual proxy.
 *
 * @author Jessica S. Reuter
 */
public abstract class InternalWindow
	extends JInternalFrame implements ActionListener {
	private static final long serialVersionUID = 20120802;
	final AppMainFrame mainFrame = AppMainFrame.getFrame();
	final JDesktopPane desktop = mainFrame.desktop;

	/**
	 * The back end calculator that's linked to this window.
	 */
	protected final RNAstructureBackendCalculator backend;

	/**
	 * The menu bar that's attached to this window.
	 */
	protected DynamicMenuBar menuBar;

	/**
	 * Constructor.
	 */
	protected InternalWindow() {
		// Initialize the back end calculator and the dialog handler.
		backend = new RNAstructureBackendCalculator();

		// Set the frame defaults.
		setDefaultCloseOperation( JInternalFrame.DISPOSE_ON_CLOSE );
		setClosable( true );
		setFocusable( true );
		setIconifiable( false );
		setMaximizable( false );
		setResizable( false );

		// Add the window to the base frame.
		desktop.add( this );

		// Set the frame icon, if the look and feel allows for it.
		// If the OS is anything but a Mac, a frame icon can be set.
		if( !OSInfo.isMac() )
		    setFrameIcon( new ImageIcon( mainFrame.getIconImage() ) );

		// Set the menu bar and menus for this window.
		menuBar = new DynamicMenuBar( this, setMenus() );

		// Add a focus listener so the window sets this window's associated
		// title and menu bar in the main frame.
		final JInternalFrame internal = this;
		addFocusListener( new FocusAdapter() {
			public void focusGained( FocusEvent e ) {
				AppMainFrame frame =
					(AppMainFrame)getDesktopPane()
					.getTopLevelAncestor();
				setTitles( getTitle() );
				frame.setJMenuBar( menuBar );
                menuBar.revalidate();
                menuBar.repaint();
				frame.repaint();
				desktop.getDesktopManager().activateFrame( internal );
			}
		});

		// Add a window closing listener that resets the main frame to its
		// original state when no more windows are open.
		addInternalFrameListener( new InternalFrameAdapter() {
			public void internalFrameClosed( InternalFrameEvent e ) {
				if( mainFrame.getMostRecentFrame() == null ) {
					mainFrame.resetFrame();
				}
			}
		});
	}

	@Override
	public void actionPerformed( ActionEvent e ) {
		Object source = e.getSource();
		Component c = source instanceof Component ? (Component)source : null;
		doActions( e.getActionCommand(), c, e );
	}

	/**
	 * Do any actions specific to this window.
	 *
	 * @param command   The command that signifies a particular action.
	 * @param sender
	 * @param event
	 */
	protected abstract void doActions(String command, final Component sender, final ActionEvent event);

	/**
	 * Create the array of variable menus used by this window.
	 *
	 * @return   The array of variable menus created for this window.
	 */
	protected abstract RolloverMenu[] setMenus();

	/**
	 * Set the title of both this window and the main window.
	 *
	 * @param title   The new title.
	 */
	protected void setTitles( String title ) {
		setTitle(title);
		AppMainFrame.getFrame().setTitle("RNAstructure -- " + title);
	}

	/**
	 * View the window.
	 */
	public void viewWindow() {
		pack();
		setLocation(0, 0);
		setVisible(true);
		menuBar.revalidate();
		menuBar.repaint();
	}
}
